
function demo_leb_disk

%--------------------------------------------------------------------------
% OBJECT:
%--------------------------------------------------------------------------
% Domain: unit-disk "B((0,0),1)".
%
% 1. Extraction of
%      a) AFP (Approximate Fekete Points)
%      b) DLP (Discrete Leja Points)
%    on the domain.
% 2. Determination of
%      c) ALP set on the disk (low Lebesgue constant)
%      d) Carnicer-Godes set
%      e) pointset based on Halton points
% 3. Evaluation of a certified Lebesgue constant on such a domain for
%    the pointsets above.
%
% The demo requires approximatively 162 seconds on a MacBook Pro with M1
% processor and 16GB of RAM.
%--------------------------------------------------------------------------
% AUTHORS
%--------------------------------------------------------------------------
% Leokadia Białas-Cież, Dimitri J. Kenne, Alvise Sommariva and Marco
% Vianello,
% Written: October 2014,
% Modified: November 22, 2023.
%--------------------------------------------------------------------------
% LICENSE
%--------------------------------------------------------------------------
% Copyright (C) 2023 Leokadia Białas-Cież, Dimitri Jordan Kenne, Alvise
% Sommariva, Marco Vianello.
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%
% Authors:
%
% L. Leokadia Białas-Cież,
% D.J. Kenne
% Alvise Sommariva <alvise@math.unipd.it>
% Marco Vianello   <marcov@math.unipd.it>
%
% Date: November 22, 2023
%--------------------------------------------------------------------------

% settings
warning off;

diary on;
diary('numerical_experiments.txt')

degV=1:25;
m=4;

% numerical experiments

lebV=[]; lebVenh=[]; C=[];
pts_str={'AFP','DLP','ALP','CG','HAL'};

set_AFP={}; set_DLP={};

for deg=degV
    [AM,CL]=AM_disk(deg,m);
    [AMH,CLH]=AM_disk(deg,4*m);
    C=[C; CL];
    for k=1:5
        switch k
            case 1
                % 'AFP';
                pts = dexsets_disk(deg,AM,0);
                set_AFP{end+1}=pts;

            case 2
                % 'DLP';
                pts = dexsets_disk(deg,AM,1);
                set_DLP{end+1}=pts;

            case 3
                % 'ALP';
                if (deg <= 30)
                    pts=set_disk_lebesgue(deg);
                else
                    pts=[];
                end

            case 4
                % 'CG';
                if (deg <= 30)
                    pts=set_disk_cg(deg);
                else
                    pts=[];
                end

            case 5
                % 'HAL';
                P = haltonset(2);
                X = net(P,10*(deg+1)*(deg+2)/2);
                iXD=find( (X(:,1)).^2+(X(:,2)).^2 <= 1);
                dim=(deg+1)*(deg+2)/2;
                pts=X(iXD(1:dim),:);

        end

        if (k == 1) | (k == 2) | ( (k == 3) & (deg <= 30)) | ...
                ( (k == 4) & (deg <= 30)) | (k == 5)
            lebL(k)=leb_eval(deg,pts,AM);
            lebH(k)=leb_eval(deg,pts,AMH);
            RE(k)=abs(lebL(k)-lebH(k))/abs(lebH(k));
        else
            lebL(k)=NaN;
            lebH(k)=NaN;
            RE(k)=NaN;
        end

    end

    lebV=[lebV; lebL lebH];
    lebVenh=[lebVenh; ((1+CL)/2)*lebL ((1+CLH)/2)*lebH];

    fprintf('\n \t .....................................................');
    fprintf('\n \t Degree   : %3.0f',deg);
    fprintf('\n \t m        : %3.0f \n',m);
    fprintf('\n \t Card AM  : %3.0f',length(AM));
    fprintf('\n \t Card AMH : %3.0f \n',length(AMH));
    fprintf('\n \t RE L Est.: %1.1e',(CL-1)/2);
    fprintf('\n \t RE H Est.: %1.1e \n',(CLH-1)/2);

    print_lebconst('AFP',lebL(1),lebH(1),CL,CLH);
    print_lebconst('DLP',lebL(2),lebH(2),CL,CLH);
    print_lebconst('ALP',lebL(3),lebH(3),CL,CLH);
    print_lebconst('Carnicer-Godes',lebL(4),lebH(4),CL,CLH);
    print_lebconst('Halton',lebL(4),lebH(4),CL,CLH);
    fprintf('\n \t .....................................................');
    fprintf('\n')

end

diary off;

% Save pointsets.

saveset(degV,set_AFP,'set_AFP_disk.m');
saveset(degV,set_DLP,'set_DLP_disk.m');

% Plots

% 1. All sets.
clear_figure(1)
fig=figure(1);
semilogy(degV',lebVenh(:,1),'r:','LineWidth',4);
hold on;
semilogy(degV(:),lebVenh(:,2),'b--','LineWidth',4);
semilogy(degV',lebVenh(:,3),'c-.','LineWidth',4);
semilogy(degV',lebVenh(:,4),'m-d','LineWidth',4);
semilogy(degV',lebVenh(:,5),'g-*','LineWidth',4);
legend(pts_str{1},pts_str{2},pts_str{3},pts_str{4},pts_str{5},'Location','northwest');
grid on;
hold on;
print(fig,'leb_disk1','-depsc');
savefig('leb_disk1.fig');

% 2. Best sets.
clear_figure(2)
fig=figure(2);
semilogy(degV',lebVenh(:,1),'r:','LineWidth',4);
hold on;
semilogy(degV(:),lebVenh(:,2),'b--','LineWidth',4);
semilogy(degV',lebVenh(:,3),'c-.','LineWidth',4);
semilogy(degV',lebVenh(:,4),'m-d','LineWidth',4);
legend(pts_str{1},pts_str{2},pts_str{3},pts_str{4},'Location','northwest');
grid on;
hold on;
print(fig,'leb_disk2','-depsc');
savefig('leb_disk2.fig');





function leb_constant=leb_eval(deg,pts,AM)

%--------------------------------------------------------------------------
% OBJECT:
%--------------------------------------------------------------------------
% computes the Lebesgue constant "leb_constant" of the pointset defined by
% "pts" with "deg+1" points, using the admissible mesh "AM".
%--------------------------------------------------------------------------
% DATA:
%--------------------------------------------------------------------------
% Authors: Alvise Sommariva and Marco Vianello, University of Padova
% Written: October 2014
% Modified: November 2023.
%--------------------------------------------------------------------------


if isempty(pts), error('pts is empty in leb_eval'); end

Vpts=vandermonde_logan_shepp(deg,pts);
VAM=vandermonde_logan_shepp(deg,AM);
leb_constant=norm(Vpts'\VAM',1);




function print_lebconst(str,lebL,lebH,CL,CH)

fprintf('\n \t '); disp(str);
fprintf('\n \t Interval        : [%1.5e,%1.5e]',lebL(1),CL*lebL(1));
fprintf('\n \t Leb.Const.(low) : %1.5e',(1+CL)/2*lebL(1));
fprintf('\n \t Leb.Const.(high): %1.5e',(1+CH)/2*lebH(1));
relerr=abs((1+CH)/2*lebH(1)-(1+CL)/2*lebL(1))/abs((1+CL)/2*lebL(1));
fprintf('\n \t rel. error    : %1.5e',relerr);
if relerr > (CL-1)/2,
    fprintf(2,'\n \t Estimate not valid');
end
fprintf('\n');






function clear_figure(nfig)

h=figure(nfig);
f_nfig=ishandle(h)&&strcmp(get(h,'type'),'figure');
if f_nfig,
    clf(nfig);
end


